/*
 * thread-tracker.h - Tracks threads in a GstPipeline
 * Copyright (C) 2009-2010 Collabora Ltd.
 * Copyright (C) 2009-2010 Nokia Corporation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#ifndef __THREAD_TRACKER_H__
#define __THREAD_TRACKER_H__

#include <gst/gst.h>

G_BEGIN_DECLS

typedef void (*setup_thread_prio) (pthread_t thread, pid_t tid);

struct ThreadTrackingData {
  GMutex *mutex;

  GHashTable *object_threads;
  GHashTable *object_tids;

  GPtrArray *otds;

  setup_thread_prio restore_thread_prio;
};

struct ObjectTrackingData {
  struct ThreadTrackingData *ttd;
  GPtrArray *objects;

  setup_thread_prio setup_thread_prio;
};

void thread_tracking_init (struct ThreadTrackingData *ttd,
    GMutex *mutex,
    setup_thread_prio restore_thread_prio);
void object_tracking_init (struct ThreadTrackingData *ttd,
    struct ObjectTrackingData *otd,
    setup_thread_prio setup_thread_prio);

void thread_tracking_data_free (struct ThreadTrackingData *ttd);
void object_tracking_data_free (struct ObjectTrackingData *otd);

void object_tracker_add_sink (struct ObjectTrackingData *otd, GstElement *sink);
void object_tracker_add_src (struct ObjectTrackingData *otd, GstElement *src);

void enter_thread (struct ThreadTrackingData *ttd, GstObject *src);
void leave_thread (struct ThreadTrackingData *ttd, GstObject *src,
    GstElement *owner);

G_END_DECLS

#endif /* __THREAD_TRACKER_H__ */
